// Copyright 2006 Nemanja Trifunovic

/*
Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/


#ifndef UTF8_FOR_CPP_2675DCD0_9480_4c0c_B92A_CC14C027B731
#define UTF8_FOR_CPP_2675DCD0_9480_4c0c_B92A_CC14C027B731

#include <string>
#include "3rdparty/utf8/checked.h"
#include "3rdparty/utf8/unchecked.h"


namespace utf8 {
	inline std::string w2a(const std::wstring& w) {
		std::string a;
		utf8::utf16to8(w.begin(), w.end(), std::back_inserter(a));
		return a;
	}

	inline std::wstring a2w(const std::string& a) {
		std::wstring w;
		utf8::utf8to16(a.begin(), a.end(), std::back_inserter(w));
		return w;
	}



	inline bool mbcs_to_u16(const char* mbcs, wchar_t* u16buffer, size_t u16size) {
		size_t request_size = MultiByteToWideChar(CP_ACP, 0, mbcs, -1, NULL, 0);
		if (0 < request_size && request_size < u16size) {
			MultiByteToWideChar(CP_ACP, 0, mbcs, -1, u16buffer, request_size);
			return true;
		}
		return false;
	};

	inline std::wstring mbcs_to_u16(const std::string& mbcs) {
		std::wstring res = L"";
		size_t request_size = MultiByteToWideChar(CP_ACP, 0, mbcs.c_str(), -1, NULL, 0);
		if (0 < request_size) {
			auto u16buffer = new wchar_t[request_size];
			MultiByteToWideChar(CP_ACP, 0, mbcs.c_str(), -1, u16buffer, request_size);
			res = u16buffer;
			delete[] u16buffer;
		}
		return res;
	}

	inline std::string u16_to_mbcs(const std::wstring& u16) {
		std::string res = "";
		size_t request_size = WideCharToMultiByte(CP_ACP, 0, u16.c_str(), -1, 0, 0, 0, 0);
		if (0 < request_size) {
			auto mbcs_buffer = new char[request_size];
			WideCharToMultiByte(CP_ACP, 0, u16.c_str(), -1, mbcs_buffer, request_size, 0, 0);
			res = mbcs_buffer;
			delete[] mbcs_buffer;
		}
		return res;
	}


	inline std::string mbcs_to_utf8(const std::wstring& mbcs) {
		std::string res = "";
		size_t request_size = WideCharToMultiByte(CP_UTF8, 0, mbcs.c_str(), -1, 0, 0, 0, 0);
		if (0 < request_size) {
			auto mbcs_buffer = new char[request_size];
			WideCharToMultiByte(CP_UTF8, 0, mbcs.c_str(), -1, mbcs_buffer, request_size, 0, 0);
			res = mbcs_buffer;
			delete[] mbcs_buffer;
		}
		return res;
	}

}

#endif // header guard
